@@ -14,7 +14,7 @@ Windows netsh utility's wlan context
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =head1 SYNOPSIS
 
@@ -35,20 +35,35 @@ use Win32::Netsh::Utils qw(:all);
 use Data::Dumper;
 use Exporter::Easy (
   EXPORT => [],
-  OK =>
-    [qw(wlan_interface_info_all wlan_interface_info wlan_profile_info_all wlan_profile_info wlan_debug)],
+  OK     => [
+    qw(
+      wlan_connect
+      wlan_disconnect
+      wlan_interface_info_all
+      wlan_interface_info
+      wlan_profile_info_all
+      wlan_profile_info
+      wlan_debug
+      )
+  ],
   TAGS => [
+    connect => [qw(wlan_connect wlan_disconnect),],
     debug   => [qw(wlan_debug),],
     profile => [
-      qw(wlan_profile_info_all wlan_profile_info wlan_profile_add wlan_profile_delete)
+      qw(
+        wlan_profile_info_all
+        wlan_profile_info
+        wlan_profile_add
+        wlan_profile_delete
+        )
     ],
     interface => [qw(wlan_interface_info_all wlan_interface_info)],
-    all       => [qw(wlan_last_error :debug :profile :interface),],
+    all       => [qw(wlan_last_error :debug :connect :profile :interface),],
   ],
 );
 
 ## Version string
-our $VERSION = qq{0.01};
+our $VERSION = qq{0.02};
 
 my $debug = 0;
 
@@ -82,6 +97,7 @@ Readonly::Scalar my $WLAN_PROFILE_KEY_LOOKUP => {
   qq{Cipher}          => qq{cipher},
   qq{Connection mode} => qq{mode},
   qq{SSID name}       => qq{ssid},
+  qq{SSID names}      => qq{ssid},
 };
 
 ##****************************************************************************
@@ -227,6 +243,8 @@ sub wlan_interface_info_all
   my $interfaces = [];
   my $interface;
 
+  print(qq{wlan_interface_info_all()\n}) if ($debug);
+
   my $command  = qq{wlan show interface};
   my $response = netsh($command);
   if ($debug >= 2)
@@ -387,14 +405,21 @@ Signal strength as a percentage
 sub wlan_interface_info
 {
   my $name = shift;
-  
-  return unless($name);
 
   print(qq{wlan_interface_info("$name")\n}) if ($debug);
 
+  ## Make sure a name was provided
+  unless ($name)
+  {
+    ## Set the module error message
+    $wlan_error = qq{No name provided!};
+    print($wlan_error, qq{\n}) if ($debug >= 2);
+    return;
+  }
+
   ## Reset the module error message
   $wlan_error = qq{};
-  
+
   foreach my $interface (@{wlan_interface_info_all()})
   {
     if (uc($name) eq uc($interface->{name}))
@@ -403,10 +428,10 @@ sub wlan_interface_info
       {
         print(Data::Dumper->Dump([$interface,], [qw(interface),]), qq{\n});
       }
-      return($interface);
+      return ($interface);
     }
   }
-  
+
   ## Set the module error message
   $wlan_error = qq{Could locate wireless interface "$name"};
   print($wlan_error, qq{\n}) if ($debug >= 2);
@@ -459,7 +484,7 @@ Name of the interface
 
 =item I<ssid>
 
-SSID of the wireless network
+Array reference to the list of SSIDs of the wireless network
 
 =item I<net_type>
 
@@ -492,6 +517,17 @@ sub wlan_profile_info
 {
   my $name = shift // qq{};
 
+  print(qq{wlan_profile_info("$name")\n}) if ($debug);
+
+  ## Make sure a name was provided
+  unless ($name)
+  {
+    ## Set the module error message
+    $wlan_error = qq{No name provided!};
+    print($wlan_error, qq{\n}) if ($debug >= 2);
+    return;
+  }
+
   my $command  = qq{wlan show profile name="$name"};
   my $response = netsh($command);
   if ($debug >= 2)
@@ -537,7 +573,7 @@ Name of the interface
 
 =item I<ssid>
 
-SSID of the wireless network
+Array reference to the list of SSIDs of the wireless network
 
 =item I<net_type>
 
@@ -611,6 +647,7 @@ sub _parse_profile_info
 {
   my $lines = shift // [];
   my $info;
+  my $last_key = qq{};
 
   print(qq{_parse_profile_info()\n}) if ($debug);
   print(Data::Dumper->Dump([$lines,], [qw(lines),]), qq{\n}) if ($debug >= 2);
@@ -633,7 +670,9 @@ PROFILE_INFO_PARSE_LOOP:
       }
 
       ## Initialize the hash
+      print(qq{  INITIALIZING HASH\n}) if ($debug);
       $info = initialize_hash_from_lookup($WLAN_PROFILE_KEY_LOOKUP);
+      $info->{ssid} = [];
       $info->{name} = $1;
       $info->{interface} = $2;
     }
@@ -641,16 +680,30 @@ PROFILE_INFO_PARSE_LOOP:
     {
       my $text  = str_trim($1);
       my $value = str_trim($2);
-      print(qq{  TEXT:  [$text]\n  VALUE: [$value]\n}) if ($debug);
-
+      print(qq{  + TEXT:  [$text]\n  + VALUE: [$value]\n}) if ($debug);
       if (my $key = get_key_from_lookup($text, $WLAN_PROFILE_KEY_LOOKUP))
       {
+        print(qq{  + KEY:   [$key]\n}) if ($debug);
         ## Remove the enclosing " for the SSID
         $value = substr($value, 1, -1) if ($key eq qq{ssid});
         $value = substr($value, 8, -2) if ($key eq qq{mode});
-        $info->{$key} = $value;
+        if (ref($info->{$key}) eq qq{ARRAY})
+        {
+          push(@{$info->{$key}}, $value);
+        }
+        else
+        {
+          $info->{$key} = $value;
+        }
+        $last_key = $key;
       }
     }
+    elsif (($last_key eq qq{ssid}) and ($line =~ /"([^"]*)"/x))
+    {
+      my $ssid = $1;
+      print(qq{  + VALUE: [$ssid]\n  + KEY:   [$last_key]\n}) if ($debug);
+      push(@{$info->{$last_key}}, $ssid);
+    }
   }
 
   if ($debug >= 2)
@@ -709,9 +762,20 @@ UNDEF on error, or 1 for success
 ##----------------------------------------------------------------------------
 sub wlan_profile_add
 {
-  my $filename = shift;
-  my $options = shift // {};
-  
+  my $filename = shift // qq{};
+  my $options  = shift // {};
+
+  print(qq{wlan_profile_add("$filename")\n}) if ($debug);
+
+  ## Make sure a name was provided
+  unless ($filename)
+  {
+    ## Set the module error message
+    $wlan_error = qq{No filename provided!};
+    print($wlan_error, qq{\n}) if ($debug >= 2);
+    return;
+  }
+
   ## Reset the module error message
   $wlan_error = qq{};
 
@@ -768,11 +832,20 @@ sub wlan_profile_delete
 {
   my $name = shift;
 
+  print(qq{wlan_profile_delete("$name")\n}) if ($debug);
+
+  ## Make sure a name was provided
+  unless ($name)
+  {
+    ## Set the module error message
+    $wlan_error = qq{No name provided!};
+    print($wlan_error, qq{\n}) if ($debug >= 2);
+    return;
+  }
+
   ## Reset the module error message
   $wlan_error = qq{};
 
-  return unless (defined($name));
-
   my $command  = qq{wlan delete profile name="$name"};
   my $response = netsh($command);
   if ($debug >= 2)
@@ -799,7 +872,7 @@ sub wlan_profile_delete
 
 =item B<Description>
 
-Return the error string associated with the last command
+Return the error string associated with the last wlan command
 
 =item B<Parameters>
 
@@ -807,7 +880,7 @@ NONE
 
 =item B<Return>
 
-SCALAR - Error string
+SCALAR - error string
 
 =back
 
@@ -819,10 +892,131 @@ sub wlan_last_error
   return ($wlan_error);
 }
 
+##****************************************************************************
+##****************************************************************************
+
+=head2 wlan_connect($name)
+
+=over 2
+
+=item B<Description>
+
+Connect to the wireless network specified in the named profile
+
+=item B<Parameters>
+
+=over 4
+
+=item I<$name>
+
+Name of the profile to use to connect.
+
+=back
+
+=item B<Return>
+
+UNDEF on error, or 1 for success
+
+=back
+
+=cut
+
+##----------------------------------------------------------------------------
+sub wlan_connect
+{
+  my $name = shift // qq{};
+
+  print(qq{wlan_connect("$name")\n}) if ($debug);
+
+  ## Make sure a name was provided
+  unless ($name)
+  {
+    ## Set the module error message
+    $wlan_error = qq{No name provided!};
+    print($wlan_error, qq{\n}) if ($debug >= 2);
+    return;
+  }
+
+  ## Reset the module error message
+  $wlan_error = qq{};
+
+  my $command  = qq{wlan connect name="$name"};
+  my $response = netsh($command);
+  if ($debug >= 2)
+  {
+    print(qq{COMMAND:  [netsh $command]\n});
+    print(qq{RESPONSE: [$response]\n});
+  }
+
+  return (1) if ($response =~ /successfully/x);
+
+  ## Set the module error message
+  $wlan_error = str_trim($response);
+  print($wlan_error, qq{\n}) if ($debug >= 2);
+
+  return;
+}
+
+##****************************************************************************
+##****************************************************************************
+
+=head2 wlan_disconnect()
+
+=over 2
+
+=item B<Description>
+
+Disconnect any current connection
+
+=item B<Parameters>
+
+=over 4
+
+=item I<NONE>
+
+=back
+
+=item B<Return>
+
+=over 4
+
+=item I<NONE>
+
+=back
+
+=back
+
+=cut
+
+##----------------------------------------------------------------------------
+sub wlan_disconnect
+{
+  print(qq{wlan_disconnect()\n}) if ($debug);
+
+  ## Reset the module error message
+  $wlan_error = qq{};
+
+  my $command  = qq{wlan disconnect};
+  my $response = netsh($command);
+  if ($debug >= 2)
+  {
+    print(qq{COMMAND:  [netsh $command]\n});
+    print(qq{RESPONSE: [$response]\n});
+  }
+
+  return;
+}
+
+
 ##****************************************************************************
 ## Additional POD documentation
 ##****************************************************************************
 
+=head1 SEE ALSO
+
+L<Win32::Netsh::Interface> for examining and controlling the netsh interface
+context including interface ipv4.
+
 =head1 AUTHOR
 
 Paul Durden E<lt>alabamapaul AT gmail.comE<gt>
